#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: caktux
# @Date:   2014-12-21 12:44:20
# @Last Modified by:   caktux
# @Last Modified time: 2014-12-22 00:48:24

import logging

import api
import json
import yaml
import config as c
config = c.read_config()

logger = logging.getLogger(__name__)

from serpent import compile

def deploy(filename, gas=config.get('deploy', 'gas'), gas_price=config.get('deploy', 'gas_price'), endowment=0, wait=False):
    # Load YAML definitions
    definitions = load_yaml(filename)

    # Dynamic variables
    variables = {}

    logger.info("Parsing %s..." % filename)

    for definition in definitions:
        for key in definition:
            logger.info("%s: " % key)

            if key == 'set':
                for name in definition[key]:
                    variables[name] = definition[key][name]
                    logger.info("  %s: %s" % (name, definition[key][name]))

            if key == 'deploy':
                for name in definition[key]:
                    for option in definition[key][name]:
                        if option == 'contract':
                            contract = definition[key][name][option]
                        if option == 'gas':
                            gas = definition[key][name][option]
                        if option == 'gas_price':
                            gas_price = definition[key][name][option]
                        if option == 'endowment':
                            endowment = definition[key][name][option]
                        if option == 'wait':
                            wait = definition[key][name][option]
                    logger.info("    Deploying %s..." % contract)
                    create(contract, gas, gas_price, endowment, wait)


def create(contract, gas, gas_price, endowment, wait):
    instance = api.Api()
    contract = compile(open(contract).read()).encode('hex')
    contract_address = instance.create(contract)
    logger.info("      Contract is available at %s" % contract_address)
    if wait:
        instance.wait_for_next_block(verbose=True)


def load_yaml(filename):
    logger.info("Loading %s..." % filename)
    f = open(filename)
    data = yaml.load(f)
    f.close()
    logger.debug(json.dumps(data, indent=4))

    return data




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: caktux
# @Date:   2014-12-21 12:44:20
# @Last Modified by:   caktux
# @Last Modified time: 2014-12-22 00:34:32

import os
import uuid
import StringIO
import ConfigParser
from utils import config_dir

def default_config_dir():
    config_dir._set_default()
    return config_dir.path

def default_config_path():
    return os.path.join(default_config_dir(), 'config')


config_template = \
"""
[api]
# JSONRPC host and port
host = 127.0.0.1
port = 8080
address = cd2a3d9f938e13cd947ec05abc7fe734df8dd826

[deploy]
gas = 10000
gas_price = 10000000000000

[misc]
config_dir = {0}
verbosity = 1

# :INFO, :WARN, :DEBUG, pyepm.deploy:DEBUG ...
logging = :INFO
""".format(default_config_dir())


def get_default_config():
    f = StringIO.StringIO()
    f.write(config_template)
    f.seek(0)
    config = ConfigParser.ConfigParser()
    config.readfp(f)
    return config

def read_config(cfg_path=default_config_path()):
    # create default if not existent
    if not os.path.exists(cfg_path):
        open(cfg_path, 'w').write(config_template)
    # extend on the default config
    config = get_default_config()
    config.read(cfg_path)
    return config

def dump_config(config):
    r = ['']
    for section in config.sections():
        for a, v in config.items(section):
            r.append('[%s] %s = %r' % (section, a, v))
    return '\n'.join(r)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: jorisbontje
# @Date:   2014-08-03 13:53:04
# @Last Modified by:   caktux
# @Last Modified time: 2014-12-22 00:53:01

import json
import logging
import requests
import sys
import time
from uuid import uuid4

from pyethereum import utils
import serpent

import config as c
config = c.read_config()

logger = logging.getLogger(__name__)

class ApiException(Exception):
    def __init__(self, code, message):
        self.code = code
        self.message = message

    def __str__(self):
        return "code=%d, message=\"%s\"" % (self.code, self.message)


class Api(object):

    def __init__(self):
        self.host = config.get('api', 'host')
        self.port = config.getint('api', 'port')
        self.jsonrpc_url = "http://%s:%s" % (self.host, self.port)
        self.address = config.get("api", "address")

    def _rpc_post(self, method, params):
        payload = {
            "jsonrpc": "2.0",
            "id": str(uuid4()),
            "method": method,
            "params": params}
        headers = {'content-type': 'application/json'}

        r = requests.post(self.jsonrpc_url, data=json.dumps(payload), headers=headers)
        if r.status_code >= 400:
            raise ApiException(r.status_code, r.reason)

        response = r.json()

        logger.debug(response)

        if 'error' in response:
            raise ApiException(response['error']['code'], response['error']['message'])

        return response.get('result')

    def accounts(self):
        return self._rpc_post('eth_accounts', None)

    def balance_at(self, address):
        params = [address]
        balance = self._rpc_post('eth_balanceAt', params)
        if balance == "0x":
            return 0
        else:
            return int(balance, 16)

    def block(self, nr):
        params = [nr]
        return self._rpc_post('eth_blockByNumber', params)

    def check(self, addresses):
        params = {
            'a': addresses
        }
        return self._rpc_post('check', params)

    def coinbase(self):
        return self._rpc_post('eth_coinbase', None)

    def create(self, code, from_=config.get("api", "address"), gas=config.get("deploy", "gas"), gas_price=config.get("deploy", "gas_price"), value=0):
        if not code.startswith('0x'):
            code = '0x' + code
        params = [{'code': code}]
        # FIXME
        # params = [{
        #    'code': code,
        #    'from': from_,
        #    'gas': hex(gas),
        #    'gasPrice': hex(gas_price),
        #    'value': hex(value)}]
        return self._rpc_post('eth_transact', params)

    def is_contract_at(self, address):
        params = [address]
        return int(self._rpc_post('eth_codeAt', params), 16) != 0

    def is_listening(self):
        return self._rpc_post('eth_listening', None)

    def is_mining(self):
        return self._rpc_post('eth_mining', None)

    def key(self):
        return self._rpc_post('key', None)

    def keys(self):
        return self._rpc_post('keys', None)

    def last_block(self):
        return self.block(-1)

    def lll(self, contract):
        params = {
            's': contract
        }
        return self._rpc_post('eth_lll', params)

    def number(self):
        return self._rpc_post('eth_number', None)

    def peer_count(self):
        return self._rpc_post('eth_peerCount', None)

    def state_at(self, address, index):
        params = [address, index]
        return self._rpc_post('eth_stateAt', params)

    def storage_at(self, address):
        params = [address]
        return self._rpc_post('eth_storageAt', params)

    def transact(self, dest, from_=config.get("api", "address"), funid=None, data="", gas=config.get("deploy", "gas"), gas_price=config.get("deploy", "gas_price"), value=0):
        if not dest.startswith('0x'):
            dest = '0x' + dest

        if funid is not None:
            data = "0x" + serpent.encode_abi(funid, data).encode('hex')

        params = [{
            'to': dest,
            'data': data,
            'gas': hex(gas),
            'gasPrice': hex(gas_price),
            'value': hex(value)}]
        return self._rpc_post('eth_transact', params)

    def call(self, dest, from_=config.get("api", "address"), funid=None, data="", gas=config.get("deploy", "gas"), gas_price=config.get("deploy", "gas_price"), value=0):
        if not dest.startswith('0x'):
            dest = '0x' + dest

        if funid is not None:
            data = "0x" + serpent.encode_abi(funid, data).encode('hex')

        params = [{
            'to': dest,
            'data': data,
            'from': from_,
            'gas': hex(gas),
            'gasPrice': hex(gas_price),
            'value': hex(value)}]
        r = self._rpc_post('eth_call', params)
        return serpent.decode_datalist(r[2:].decode('hex'))

    def wait_for_next_block(self, verbose=False):
        if verbose:
            sys.stdout.write('Waiting for next block to be mined')
            start_time = time.time()

        last_block = self.last_block()
        while True:
            if verbose:
                sys.stdout.write('.')
                sys.stdout.flush()
            time.sleep(5)
            block = self.last_block()
            if block != last_block:
                break

        if verbose:
            delta = time.time() - start_time
            logger.info("Ready! Mining took %ds" % delta)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: caktux
# @Date:   2014-12-21 12:44:20
# @Last Modified by:   caktux
# @Last Modified time: 2014-12-22 00:24:52

import os
import sys
import pprint
import logging
import logging.config

class ConfigDir(object):

    ethdirs = {
        "linux2": "~/.pyepm",
        "darwin": "~/.pyepm",
        "win32": "~/AppData/Roaming/PyEPM",
        "win64": "~/AppData/Roaming/PyEPM",
    }

    def __init__(self):
        self._path = None

    def set(self, path):
        path = os.path.abspath(path)
        if not os.path.exists(path):
            os.makedirs(path)
        assert os.path.isdir(path)
        self._path = path

    def _set_default(self):
        p = self.ethdirs.get(sys.platform, self.ethdirs['linux2'])
        self.set(os.path.expanduser(os.path.normpath(p)))

    @property
    def path(self):
        if not self._path:
            self._set_default()
        return self._path

def configure_logging(loggerlevels=':DEBUG', verbosity=1):
    logconfig = dict(
        version=1,
        disable_existing_loggers=False,
        formatters=dict(
            debug=dict(
                format='%(message)s' # '%(threadName)s:%(module)s: %(message)s'
            ),
            minimal=dict(
                format='%(message)s'
            ),
        ),
        handlers=dict(
            default={
                'level': 'INFO',
                'class': 'logging.StreamHandler',
                'formatter': 'minimal'
            },
            verbose={
                'level': 'DEBUG',
                'class': 'logging.StreamHandler',
                'formatter': 'debug'
            },
        ),
        loggers=dict()
    )

    for loggerlevel in filter(lambda _: ':' in _, loggerlevels.split(',')):
        name, level = loggerlevel.split(':')
        logconfig['loggers'][name] = dict(
            handlers=['verbose'], level=level, propagate=False)

    if len(logconfig['loggers']) == 0:
        logconfig['loggers'][''] = dict(
            handlers=['default'],
            level={0: 'ERROR', 1: 'WARNING', 2: 'INFO', 3: 'DEBUG'}.get(
                verbosity),
            propagate=True)

    logging.config.dictConfig(logconfig)
    logging.debug("Logging config: \n%s\n=====" % pprint.pformat(logconfig, width=4))

config_dir = ConfigDir()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: caktux
# @Date:   2014-12-21 08:28:41
# @Last Modified by:   caktux
# @Last Modified time: 2014-12-21 11:42:56

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: caktux
# @Date:   2014-12-21 12:44:20
# @Last Modified by:   caktux
# @Last Modified time: 2014-12-22 00:16:58

import os
import json
import shutil
import config as c
import logging
import logging.config
from deploy import *
logger = logging.getLogger(__name__)

from utils import config_dir, configure_logging
from argparse import ArgumentParser

from . import __version__

def parse_arguments(parser):
    parser.add_argument(
        "-r", "--host",
        dest="host",
        help="<host>  JSONRPC host (default: 127.0.0.1).")
    parser.add_argument(
        "-p", "--port",
        dest="port",
        help="<port>  JSONRPC port (default: 8080).")
    parser.add_argument(
        "-a", "--address",
        dest="address",
        help="Set the address from which to deploy contracts.")
    parser.add_argument(
        "-g", "--gas",
        dest="gas",
        help="Set the default amount of gas for deployment.")
    parser.add_argument(
        "-c", "--config",
        dest="config",
        help="Use another configuration file.")
    parser.add_argument(
        "-V", "--verbose",
        dest="verbosity",
        help="<0 - 3> Set the log verbosity from 0 to 3 (default: 1)")
    parser.add_argument(
        "-L", "--logging",
        dest="logging",
        help="<logger1:LEVEL,logger2:LEVEL> set the console log level for"
        " logger1, logger2, etc. Empty loggername means root-logger,"
        " e.g. ':DEBUG,:INFO'. Overrides '-V'")
    parser.add_argument(
        "filename",
        nargs='+',
        help="Package definition filenames in YAML format")

    return parser.parse_args()


def create_config(parser):
    options = parse_arguments(parser)

    # 1) read the default config at "~/.pyepm"
    config = c.read_config()

    # 2) read config from file
    cfg_fn = getattr(options, 'config')
    if cfg_fn:
        if not os.path.exists(cfg_fn):
            c.read_config(cfg_fn)  # creates default
        config.read(cfg_fn)

    # 3) apply cmd line options to config
    for section in config.sections():
        for a, v in config.items(section):
            if getattr(options, a, None) is not None:
                config.set(section, a, getattr(options, a))

    # set config_dir
    config_dir.set(config.get('misc', 'config_dir'))

    return config


def main():
    config = c.get_default_config()
    parser = ArgumentParser(version=__version__)

    config = create_config(parser)

    # Logging
    configure_logging(config.get('misc', 'logging') or '',

    verbosity=config.getint('misc', 'verbosity'))
    logger.info('PyEPM %s', __version__)
    logger.info('=====')

    logger.debug(c.dump_config(config))

    args = parser.parse_args()

    for filename in args.filename:
        if not os.path.exists(filename):
            logger.warn("File does not exist: %s" % filename)
        else:
            logger.info("Deploying %s..." % filename)
            deploy(filename)


if __name__ == '__main__':
    main()




# This file was generated by 'versioneer.py' (0.12) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

version_version = '0.7.0-2-g0c0928f'
version_full = '0c0928f8ff7674bfec725ba4c873fa18a35d8e8e'
def get_versions(default={}, verbose=False):
    return {'version': version_version, 'full': version_full}




from setuptools import setup, find_packages
import versioneer
versioneer.VCS = 'git'
versioneer.versionfile_source = 'pyepm/_version.py'
versioneer.versionfile_build = 'pyepm/_version.py'
versioneer.tag_prefix = '' # tags are like 1.2.0
versioneer.parentdir_prefix = 'pyepm-' # dirname like 'myproject-1.2.0'

console_scripts = ['pyepm=pyepm.pyepm:main']

setup(name="pyepm",
      packages=find_packages("."),
      description='Python Ethereum Package Manager',
      url='https://github.com/etherex/pyepm/',
      install_requires=[
          'pyethereum',
          'ethereum-serpent',
          'requests'
      ],
      entry_points=dict(console_scripts=console_scripts),
      version=versioneer.get_version(),
      cmdclass=versioneer.get_cmdclass())
